@@ -27,6 +27,7 @@ import java.util.LinkedList;
 
 import org.apache.log4j.Logger;
 
+import mobac.gui.MainGUI;
 import mobac.gui.mapview.controller.DefaultMapController;
 import mobac.gui.mapview.controller.JMapController;
 import mobac.gui.mapview.controller.MapKeyboardController;
@@ -81,7 +82,10 @@ public class PreviewMap extends JMapViewer {
 	private int gridZoom = -1;
 	private int gridSize;
 
-	protected LinkedList<MapEventListener> mapEventListeners = new LinkedList<MapEventListener>();
+	public boolean isMeasuring = false;
+	public final Ruler ruler = new Ruler(this);
+
+	protected LinkedList<MapEventListener> mapEventListeners = new LinkedList<>();
 
 	protected JMapController mapKeyboardController;
 	protected JMapController mapSelectionController;
@@ -165,6 +169,11 @@ public class PreviewMap extends JMapViewer {
 		return gridZoom;
 	}
 
+	public void setIsMeasuring(Boolean val) {
+		isMeasuring = val;
+		MainGUI.getMainGUI().setEnableLeftPanel(!isMeasuring);
+	}
+
 	/**
 	 * Updates the <code>gridSize</code> and the <code>gridTile</code>. This method has to called if
 	 * <code>mapSource</code> or <code>zoom</code> as been changed.
@@ -238,31 +247,33 @@ public class PreviewMap extends JMapViewer {
 				}
 			}
 		}
-		if (gridSelectionStart != null && gridSelectionEnd != null) {
-			// Draw the selection rectangle widened by the current grid
-			int zoomDiff = MAX_ZOOM - zoom;
-			int x_min = (gridSelectionStart.x >> zoomDiff) - tlc.x;
-			int y_min = (gridSelectionStart.y >> zoomDiff) - tlc.y;
-			int x_max = (gridSelectionEnd.x >> zoomDiff) - tlc.x;
-			int y_max = (gridSelectionEnd.y >> zoomDiff) - tlc.y;
+		if (!isMeasuring) {
+			if (gridSelectionStart != null && gridSelectionEnd != null) {
+				// Draw the selection rectangle widened by the current grid
+				int zoomDiff = MAX_ZOOM - zoom;
+				int x_min = (gridSelectionStart.x >> zoomDiff) - tlc.x;
+				int y_min = (gridSelectionStart.y >> zoomDiff) - tlc.y;
+				int x_max = (gridSelectionEnd.x >> zoomDiff) - tlc.x;
+				int y_max = (gridSelectionEnd.y >> zoomDiff) - tlc.y;
 
-			int w = x_max - x_min + 1;
-			int h = y_max - y_min + 1;
-			g.setColor(SEL_COLOR);
-			g.fillRect(x_min, y_min, w, h);
-		}
-		if (iSelectionMin != null && iSelectionMax != null) {
-			// Draw the selection rectangle exactly as it has been specified by the user
-			int zoomDiff = MAX_ZOOM - zoom;
-			int x_min = (iSelectionMin.x >> zoomDiff) - tlc.x;
-			int y_min = (iSelectionMin.y >> zoomDiff) - tlc.y;
-			int x_max = (iSelectionMax.x >> zoomDiff) - tlc.x;
-			int y_max = (iSelectionMax.y >> zoomDiff) - tlc.y;
+				int w = x_max - x_min + 1;
+				int h = y_max - y_min + 1;
+				g.setColor(SEL_COLOR);
+				g.fillRect(x_min, y_min, w, h);
+			}
+			if (iSelectionMin != null && iSelectionMax != null) {
+				// Draw the selection rectangle exactly as it has been specified by the user
+				int zoomDiff = MAX_ZOOM - zoom;
+				int x_min = (iSelectionMin.x >> zoomDiff) - tlc.x;
+				int y_min = (iSelectionMin.y >> zoomDiff) - tlc.y;
+				int x_max = (iSelectionMax.x >> zoomDiff) - tlc.x;
+				int y_max = (iSelectionMax.y >> zoomDiff) - tlc.y;
 
-			int w = x_max - x_min + 1;
-			int h = y_max - y_min + 1;
-			g.setColor(GRID_COLOR);
-			g.drawRect(x_min, y_min, w, h);
+				int w = x_max - x_min + 1;
+				int h = y_max - y_min + 1;
+				g.setColor(GRID_COLOR);
+				g.drawRect(x_min, y_min, w, h);
+			}
 		}
 		if (mapSource instanceof MapSourceTextAttribution) {
 			MapSourceTextAttribution ta = (MapSourceTextAttribution) mapSource;
@@ -281,6 +292,11 @@ public class PreviewMap extends JMapViewer {
 			wgsGrid.paintWgsGrid(g, mapSource.getMapSpace(), tlc, zoom);
 		}
 		ScaleBar.paintScaleBar(this, g, mapSource.getMapSpace(), tlc, zoom);
+
+		// if measure tool is paused, keep the path overlay
+		if (isMeasuring || ruler.segments.size() > 0) {
+			ruler.paint(this, g, tlc, zoom);
+		}
 	}
 
 	public Bookmark getPositionBookmark() {
@@ -479,6 +495,36 @@ public class PreviewMap extends JMapViewer {
 		repaint();
 	}
 
+	public void runRuler(String action) {
+		if (action == "toggle") {
+			action = isMeasuring ? "stop" : "start";
+		}
+
+		switch (action) {
+		case "clearkb": // double clear disables measure mode (keyboard only)
+			if (ruler.segments.size() == 0) {
+				setIsMeasuring(false);
+			}
+		case "clear":
+			break;
+
+		case "start":
+			setIsMeasuring(true);
+			break;
+
+		case "stop":
+			setIsMeasuring(false);
+			return;
+
+		case "exit":
+			setIsMeasuring(false);
+			break;
+		}
+
+		ruler.segments.clear();
+		repaint();
+	}
+
 	public JMapController getMapKeyboardController() {
 		return mapKeyboardController;
 	}
