@@ -1,6 +1,7 @@
 package mobac.mapsources.loader;
 
 import java.io.File;
+import java.io.FileInputStream;
 import java.io.FileReader;
 import java.io.IOException;
 import java.io.InputStream;
@@ -31,6 +32,7 @@ import mobac.mapsources.custom.CustomCloudMade;
 import mobac.mapsources.custom.CustomLocalTileFilesMapSource;
 import mobac.mapsources.custom.CustomLocalTileSQliteMapSource;
 import mobac.mapsources.custom.CustomLocalTileZipMapSource;
+import mobac.mapsources.custom.CustomLocalAqmMapSource;
 import mobac.mapsources.custom.CustomMapSource;
 import mobac.mapsources.custom.CustomMapsforge;
 import mobac.mapsources.custom.CustomMultiLayerMapSource;
@@ -73,7 +75,8 @@ public class CustomMapSourceLoader {
 					CustomCloudMade.class, //
 					CustomLocalTileFilesMapSource.class, //
 					CustomLocalTileZipMapSource.class, //
-					CustomLocalTileSQliteMapSource.class //
+					CustomLocalTileSQliteMapSource.class, //
+					CustomLocalAqmMapSource.class
 			};
 			context = JAXBContext.newInstance(customMapClasses);
 		} catch (JAXBException e) {
@@ -127,8 +130,8 @@ public class CustomMapSourceLoader {
 		} catch (Exception e) {
 			log.error("Failed to load custom map source file \"" + mapSourceFile + "\": " + e);
 		}
-		try (FileReader reader = new FileReader(mapSourceFile)) {
-			return internalLoadMapSource(new InputSource(reader), mapSourceFile);
+		try (InputStream in = new FileInputStream(mapSourceFile)) {
+			return internalLoadMapSource(new InputSource(in), mapSourceFile);
 		}
 	}
 
